package encapsulation.metrics;

import java.util.*;

import encapsulation.model.Access;
import encapsulation.model.AccessibleMember;
import encapsulation.model.Entity;

/**
 * AccessStrategy class, defines a strategy for extracting accesses for metrics
 * calculations given an entity
 * @author Janina Voigt
 *
 */
public abstract class AccessStrategy {

	/**
	 * Gets the members that should be used in a metric calculation given a 
	 * certain entity
	 * @param entity the entity to get members from
	 * @return the members to use in the metrics calculation
	 */
	public abstract Set<AccessibleMember> getMembers(Entity entity);
	
	/**
	 * Gets the accesses that should be used in a metric calculation given a 
	 * certain entity
	 * @param entity the entity to get accesses from
	 * @return the accesses to use in the metrics calculation
	 */
	public abstract Set<Access> getAccesses(Entity entity);
	
	/**
	 * Gets a header to describe this certain metric
	 * @param entity the entity used in the metric calculation
	 * @return a header to print
	 */
	public abstract String getHeader(Entity entity);
}
