package encapsulation.metrics;

import encapsulation.export.ResultsWriter;
import encapsulation.model.Entity;


/**
 * MetricCalculator class, represents a metrics calculator that calculates some
 * sort of metric about an entity
 * @author Janina Voigt
 *
 */
public abstract class MetricCalculator {

	protected Entity entity;	//the entity to calculate the metric for
	
	/**
	 * Constructor for MetricCalculator objects
	 * @param theEntity the entity to calculate the metric for
	 * and members
	 */
	public MetricCalculator(Entity theEntity) {
		entity = theEntity;
	}
	
	/**
	 * Calculate the metric results and write them out using a ResultsWriter
	 * @param writer the ResultWriter to use to write out the metric results
	 */
	public abstract void calculateResults(ResultsWriter writer);
}
