package encapsulation.model;

import java.lang.reflect.Modifier;
import java.util.*;

/**
 * AccessibleMember class, represents a class member that can be accessed
 * @author Janina Voigt
 *
 */
public abstract class AccessibleMember extends Member{
	
	/**
	 * Constructor for AccessibleMember objects 
	 * @param theOwner the owner class of the member
	 */
	public AccessibleMember(ClassOrInterface theOwner){
		super(theOwner);
		accesses = new HashSet<Access>();
	}
	
	/**
	 * Returns the member as an accessible member
	 */
	public AccessibleMember asAccessibleMember() {
		return this;
	}
	
	/**
	 * Returns a boolean indicating if this member is public
	 * @return true if this member is public, false otherwise
	 */
	public boolean isPublic() {
		return Modifier.isPublic(getDecl().getModifier());
	}
	
	/**
	 * Returns a boolean indicating if this member is protected
	 * @return true if this member is protected, false otherwise
	 */
	public boolean isProtected() {
		return Modifier.isProtected(getDecl().getModifier());
	}
	
	/**
	 * Returns a boolean indicating if this member is package
	 * @return true if this member is package, false otherwise
	 */
	public boolean isPackage() {
		return (!Modifier.isPublic(getDecl().getModifier()) &&
				!Modifier.isProtected(getDecl().getModifier()) &&
				!Modifier.isPrivate(getDecl().getModifier()));
	}
	
	/**
	 * Returns a boolean indicating if this member is private
	 * @return true if this member is private, false otherwise
	 */
	public boolean isPrivate() {
		return Modifier.isPrivate(getDecl().getModifier());
	}
}
