package encapsulation.model;

import java.util.*;
import jst.symtab.Decl;

/**
 * Program class, represents a whole program
 * @author Janina Voigt
 *
 */
public class Program implements Entity {
	
	protected Set<ClassOrInterface> classTypes;
	
	/**
	 * Constructor for Program objects
	 */
	public Program() {
		classTypes = new HashSet<ClassOrInterface>();
	}
	
	/**
	 * Gets the name of the program
	 * @return the name of the program
	 */
	public String getName() {
		return "the whole program";
	}
	
	/**
	 * Adds a class or interface to the program
	 * @param theType the class or interface to add
	 */
	public void addClassOrInterface(ClassOrInterface theType){
		classTypes.add(theType);
	}

	/**
	 * Gets all members of the program
	 * @return all members of the program
	 */
	public Set<Member> getMembers(){
		Set<Member> allMembers = new HashSet<Member>();
		for(ClassOrInterface theType : classTypes){
			allMembers.addAll(theType.getMembers());
		}
		return allMembers;
	}
	
	/**
	 * Gets all accesses to members of the program
	 * @return all accesses to members
	 */
	public Set<Access> getAccesses(){
		Set<Access> allAccesses = new HashSet<Access>();
		for(ClassOrInterface theType : classTypes){
			allAccesses.addAll(theType.getAccesses());
		}
		return allAccesses;
	}
	
	/**
	 * Gets the AccessibleMember with the given JST decl
	 * @param decl the JST decl to search for
	 * @return the AccessibleMember with the same JST decl, or null if no 
	 * AccessibleMember was found
	 */
	public AccessibleMember getAccessibleMember(Decl decl) {
		AccessibleMember theMember = null;
		for(ClassOrInterface theType : classTypes){
			theMember = theType.getAccessibleMember(decl);
			if(theMember != null) {
				break;
			}
		}
		return theMember;
	}
}
