
package jst.symtab;
import java.io.*;
import jst.parsetree.*;
import java.util.*;
import java.lang.reflect.Modifier;
import jst.symtab.visitor.ModelVisitor;
import jst.symtab.link.*;

public class ConstructorDecl extends OperationDecl {

	public ConstructorDecl(ClassType ownerClass, JstParseTreeBranch theSource) {
    	super(ownerClass, ownerClass.getSimpleName(), theSource);

    	ownerClass.addConstructor(this);
    }
	
	public ConstructorDecl(ClassType newOwnerClass, ConstructorDecl that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerClass, that, swaps);

    	newOwnerClass.addConstructor(this);
	}

	protected void init() {
		super.init();

		int line = source == null? 0 : source.getLine();
		int column = source == null? 0 : source.getColumn();

        ScopeLink scope = new KnownDecl(this);
        ClassTypeLookup thiz = new ClassTypeLookup(scope, new StringIdentifier("this", line, column));
        setTypeRef(thiz);
	}

	public void copy(ClassType newOwnerClass, Map<TypeParameter, ActualType> swaps) {
		new ConstructorDecl(newOwnerClass, this, swaps);
	}

	public ConstructorDecl asConstructor() {
		return this;
	}
	
 	public String getCanonicalName() {
       return "CON_" + getOwnerScope().getCanonicalName().substring(4) + "." + getSignature();
    }

	public String getSignature() {
    	String signature = simpleName + "(";

		Iterator paramIter = parameters.iterator();
		while (paramIter.hasNext()) {
			ParameterDecl param = (ParameterDecl) paramIter.next();
			TypeDecl paramType = param.getType();
			signature += (paramType == null? "UNKNOWN" : paramType.getCanonicalName());
			if (paramIter.hasNext())
				signature += ",";
		}

		signature += ")";

		return signature;
    }

    public void accept(ModelVisitor visitor) {
        visitor.visitConstructorDecl(this);
    }
}
