
package jst.symtab;

import java.util.*;

import jst.parsetree.*;
import jst.symtab.visitor.ModelVisitor;

public class GenericClassType extends ClassType {

	protected List<TypeParameter> typeParameters;

	protected Map<List<ActualType>, ParameterizedClassType> parameterizedClasses;

	public GenericClassType(SourceFile ownerSourceFile, String theSimpleName, JstParseTreeBranch theSource) {
    	super(ownerSourceFile, theSimpleName, theSource);
    }

	public GenericClassType(PackageDecl ownerPackage, String theSimpleName, JstParseTreeBranch theSource) {
		super(ownerPackage, theSimpleName, theSource);
    }

	public GenericClassType(UserType ownerUserType, String theSimpleName, JstParseTreeBranch theSource) {
		super(ownerUserType, theSimpleName, theSource);
    }

	public GenericClassType(Block ownerBlock, String theSimpleName, JstParseTreeBranch theSource) {
		super(ownerBlock, theSimpleName, theSource);
    }

	public GenericClassType(FieldInitializer ownerFieldInitializer, String theSimpleName, JstParseTreeBranch theSource) {
		super(ownerFieldInitializer, theSimpleName, theSource);
    }

	public GenericClassType(SourceFile newOwnerSourceFile, GenericClassType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerSourceFile, that, swaps);

		init(that, swaps);
    }

	public GenericClassType(PackageDecl newOwnerPackage, GenericClassType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerPackage, that, swaps);

		init(that, swaps);
    }

	public GenericClassType(UserType newOwnerUserType, GenericClassType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerUserType, that, swaps);

		init(that, swaps);
    }

	public GenericClassType(Block newOwnerBlock, GenericClassType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerBlock, that, swaps);

		init(that, swaps);
    }

	public GenericClassType(FieldInitializer newOwnerFieldInitializer, GenericClassType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerFieldInitializer, that, swaps);

		init(that, swaps);
    }

	protected void init() {
		super.init();

		typeParameters = new ArrayList<TypeParameter>();
		parameterizedClasses = new HashMap<List<ActualType>, ParameterizedClassType>();
	}

	protected void init(GenericClassType that, Map<TypeParameter, ActualType> swaps) {
		for (TypeParameter thatTypeParameter : typeParameters)
			thatTypeParameter.copy(this, swaps);
    }

	public void copy(SourceFile newOwnerSourceFile, Map<TypeParameter, ActualType> swaps) {
		new GenericClassType(newOwnerSourceFile, this, swaps);
	}

	public void copy(PackageDecl newOwnerPackage, Map<TypeParameter, ActualType> swaps) {
		new GenericClassType(newOwnerPackage, this, swaps);
	}

	public void copy(UserType newOwnerUserType, Map<TypeParameter, ActualType> swaps) {
		new GenericClassType(newOwnerUserType, this, swaps);
	}

	public void copy(Block newOwnerBlock, Map<TypeParameter, ActualType> swaps) {
		new GenericClassType(newOwnerBlock, this, swaps);
	}

	public void copy(FieldInitializer newOwnerFieldInitializer, Map<TypeParameter, ActualType> swaps) {
		new GenericClassType(newOwnerFieldInitializer, this, swaps);
	}

	public void addTypeParameter(TypeParameter param) {
		typeParameters.add(param);
	}

	public TypeDecl lookupType(boolean widen, String typeName, ReferenceType looker) {

		// Linear search.
		for (TypeParameter param: typeParameters)
			if (param.getSimpleName().equals(typeName))
				return param;

		return super.lookupType(widen, typeName, looker);
	}

	public ParameterizedClassType parameterize(List<ActualType> typeArgs) {
		if (typeArgs.size() != typeParameters.size())
			return null;

		ParameterizedClassType result = parameterizedClasses.get(typeArgs);
		if (result != null)
			return result;

		Map<TypeParameter, ActualType> swaps = new HashMap<TypeParameter, ActualType>(typeParameters.size());
		for (int i = 0; i < typeParameters.size(); i++)
			swaps.put(typeParameters.get(i), typeArgs.get(i));

		return new ParameterizedClassType(this, swaps);
	}

	public void addParameterizedClass(ParameterizedClassType parameterized) {
		parameterizedClasses.put(parameterized.getActualTypes(), parameterized);
	}

    public void accept(ModelVisitor visitor) {
        visitor.visitGenericClassType(this);
    }
}



