
package jst.symtab;

import java.util.*;

import jst.parsetree.*;
import jst.symtab.visitor.ModelVisitor;

public class GenericInterfaceType extends InterfaceType {

	protected List<TypeParameter> typeParameters;

	protected Map<List<ActualType>, ParameterizedInterfaceType> parameterizedInterfaces;

	public GenericInterfaceType(SourceFile ownerSourceFile, String theSimpleName, JstParseTreeBranch theSource) {
    	super(ownerSourceFile, theSimpleName, theSource);
    }

	public GenericInterfaceType(PackageDecl ownerPackage, String theSimpleName, JstParseTreeBranch theSource) {
		super(ownerPackage, theSimpleName, theSource);
    }

	public GenericInterfaceType(UserType ownerUserType, String theSimpleName, JstParseTreeBranch theSource) {
		super(ownerUserType, theSimpleName, theSource);
    }

	public GenericInterfaceType(Block ownerBlock, String theSimpleName, JstParseTreeBranch theSource) {
		super(ownerBlock, theSimpleName, theSource);
    }

	public GenericInterfaceType(SourceFile newOwnerSourceFile, GenericInterfaceType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerSourceFile, that, swaps);
    }

	public GenericInterfaceType(PackageDecl newOwnerPackage, GenericInterfaceType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerPackage, that, swaps);
    }

	public GenericInterfaceType(UserType newOwnerUserType, GenericInterfaceType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerUserType, that, swaps);
    }

	public GenericInterfaceType(Block newOwnerBlock, GenericInterfaceType that, Map<TypeParameter, ActualType> swaps) {
		super(newOwnerBlock, that, swaps);
    }

	protected void init() {
		super.init();

		typeParameters = new ArrayList<TypeParameter>();
		parameterizedInterfaces = new HashMap<List<ActualType>, ParameterizedInterfaceType>();
	}

	private void init(GenericInterfaceType that, Map<TypeParameter, ActualType> swaps) {
		for (TypeParameter thatTypeParameter : typeParameters)
			thatTypeParameter.copy(this, swaps);
    }

	public void copy(SourceFile newOwnerSourceFile, Map<TypeParameter, ActualType> swaps) {
		new GenericInterfaceType(newOwnerSourceFile, this, swaps);
	}

	public void copy(PackageDecl newOwnerPackage, Map<TypeParameter, ActualType> swaps) {
		new GenericInterfaceType(newOwnerPackage, this, swaps);
	}

	public void copy(UserType newOwnerUserType, Map<TypeParameter, ActualType> swaps) {
		new GenericInterfaceType(newOwnerUserType, this, swaps);
	}

	public void copy(Block newOwnerBlock, Map<TypeParameter, ActualType> swaps) {
		new GenericInterfaceType(newOwnerBlock, this, swaps);
	}

	public void addTypeParameter(TypeParameter param) {
		typeParameters.add(param);
	}

	public TypeDecl lookupType(boolean widen, String typeName, ReferenceType looker) {

		// Linear search.
		for (TypeParameter param: typeParameters)
			if (param.getSimpleName().equals(typeName))
				return param;

		return super.lookupType(widen, typeName, looker);
	}

	public ParameterizedInterfaceType parameterize(List<ActualType> typeArgs) {
		if (typeArgs.size() != typeParameters.size())
			return null;

		ParameterizedInterfaceType result = parameterizedInterfaces.get(typeArgs);
		if (result != null)
			return result;

		Map<TypeParameter, ActualType> swaps = new HashMap<TypeParameter, ActualType>(typeParameters.size());
		for (int i = 0; i < typeParameters.size(); i++)
			swaps.put(typeParameters.get(i), typeArgs.get(i));

		return new ParameterizedInterfaceType(this, swaps);
	}

	public void addParameterizedInterface(ParameterizedInterfaceType parameterized) {
		parameterizedInterfaces.put(parameterized.getActualTypes(), parameterized);
	}

    public void accept(ModelVisitor visitor) {
        visitor.visitGenericInterfaceType(this);
    }
}



